// it is auto make config
#define DTK_VERSION_MAJOR 5
#define DTK_VERSION_MINOR 7
#define DTK_VERSION_PATCH 48
#define DTK_VERSION_BUILD 0
#define DTK_VERSION_STR "5.7.48"

#define DTKCORE_CLASS_DAbstractUnitFormatter
#define DTKCORE_CLASS_DBaseFileWatcher
#define DTKCORE_CLASS_DCapDir
#define DTKCORE_CLASS_DCapFile
#define DTKCORE_CLASS_DCapManager
#define DTKCORE_CLASS_DConfig
#define DTKCORE_CLASS_DConfigFile
#define DTKCORE_CLASS_DDBusExtendedAbstractInterface
#define DTKCORE_CLASS_DDBusInterface
#define DTKCORE_CLASS_DDBusSender
#define DTKCORE_CLASS_DDciFile
#define DTKCORE_CLASS_DDesktopEntry
#define DTKCORE_CLASS_DDiskSizeFormatter
#define DTKCORE_CLASS_DError
#define DTKCORE_CLASS_DExpected
#define DTKCORE_CLASS_DExportedInterface
#define DTKCORE_CLASS_DFileServices
#define DTKCORE_CLASS_DFileSystemWatcher
#define DTKCORE_CLASS_DFileWatcher
#define DTKCORE_CLASS_DFileWatcherManager
#define DTKCORE_CLASS_DLicenseInfo
#define DTKCORE_CLASS_DLog
#define DTKCORE_CLASS_DNotifySender
#define DTKCORE_CLASS_DObject
#define DTKCORE_CLASS_DObjectPrivate
#define DTKCORE_CLASS_DPathBuf
#define DTKCORE_CLASS_DPinyin
#define DTKCORE_CLASS_DRecentManager
#define DTKCORE_CLASS_DSGApplication
#define DTKCORE_CLASS_DSecureString
#define DTKCORE_CLASS_DSettings
#define DTKCORE_CLASS_DSettingsGroup
#define DTKCORE_CLASS_DSettingsOption
#define DTKCORE_CLASS_DSingleton
#define DTKCORE_CLASS_DStandardPaths
#define DTKCORE_CLASS_DSysInfo
#define DTKCORE_CLASS_DTextEncoding
#define DTKCORE_CLASS_DThreadUtils
#define DTKCORE_CLASS_DTimeUnitFormatter
#define DTKCORE_CLASS_DTrashManager
#define DTKCORE_CLASS_DUtil
#define DTKCORE_CLASS_DVtableHook
#define DTKCORE_CLASS_DtkCores
